@@ -1,9 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from simmate.utilities import get_doc_from_readme
-
-__doc__ = get_doc_from_readme(__file__)
-
 from .brions import Brions
 from .brmix import Brmix
 from .edddav import Edddav
