@@ -54,9 +54,10 @@ class Relaxation__Vasp__Matproj(VaspWorkflow):
     settings.
     """
 
+    description_doc_short = "based on pymatgen's MPRelaxSet"
+
     functional = "PBE"
     potcar_mappings = PBE_ELEMENT_MAPPINGS
-    description_doc_short = "uses Materials Project settings"
 
     incar = dict(
         ALGO="Fast",
