@@ -25,10 +25,15 @@ from simmate.database.base_data_types import (
 # automatically build out your table with many different columns. Many
 # also have a `from_toolkit` method that makes populating your database super easy
 class MyCustomTable1(Structure, Calculation):
-    pass  # nothing else is required unless you want to add custom columns/features
+    # here, we list out custom columns (so this code is all optional)
+    input_01 = table_column.FloatField(null=True, blank=True)
+    input_02 = table_column.BooleanField(null=True, blank=True)
+    output_01 = table_column.FloatField(null=True, blank=True)
+    output_02 = table_column.BooleanField(null=True, blank=True)
 
 
-# alternatively, we can make a DatabaseTable from scratch
+# alternatively, we can make a DatabaseTable from scratch that won't be
+# used by any workflow.
 class MyCustomTable2(DatabaseTable):
     custom_column_01 = table_column.IntegerField()
     custom_column_02 = table_column.FloatField(null=True, blank=True)
