@@ -547,24 +547,12 @@ class DatabaseTable(models.Model):
             "database_id": self.id,
         }
 
-    @staticmethod
-    def from_dict(source_dict: dict):
-
-        # This method can be return ANY table, so we need to import all of them
-        # here. This is a local import to prevent circular import issues.
-        from simmate.website.workflows import models as all_datatables
+    @classmethod
+    def from_dict(cls, source_dict: dict):
 
         # start by loading the datbase table, which is given as a module path
-        datatable_str = source_dict["database_table"]
-
-        # Import the datatable class -- how this is done depends on if it
-        # is from a simmate supplied class or if the user supplied a full
-        # path to the class
-        # OPTIMIZE: is there a better way to do this?
-        if hasattr(all_datatables, datatable_str):
-            datatable = getattr(all_datatables, datatable_str)
-        else:
-            datatable = import_string(datatable_str)
+        table_name = source_dict["database_table"]
+        datatable = cls.get_table(table_name)
 
         # These attributes tells us which structure to grab from our datatable.
         # The user should have only provided one -- if they gave more, we just
@@ -592,6 +580,63 @@ class DatabaseTable(models.Model):
 
         return database_object
 
+    @classmethod
+    def from_dicts(cls, source_dicts: list[dict]):
+        """
+        Given many database dictionaries, this will perform optimized database
+        queries.
+
+        This method should be preffered over from_dict when you have a many
+        entries that you want to load
+        """
+        # collect the unique sources so that we can make a single query.
+        query_info = {}  # dictionary of database table + all ids
+        for source in source_dicts:
+            table_name = source["database_table"]
+            table_id = source["database_id"]
+
+            if table_name not in query_info.keys():
+                query_info[table_name] = []
+
+            query_info[table_name].append(table_id)
+
+        results = []
+        for table_name, table_ids in query_info.items():
+            datatable = cls.get_table(table_name)
+            query = datatable.objects.filter(id__in=table_ids).all()
+            results += list(query)
+
+        # the query does not return the ids in the same order that table_ids
+        # was given. Order is important in some cases, so we fix this here.
+        all_data_dict = {entry.id: entry for entry in results}
+        all_data_ordered = [
+            all_data_dict[id]
+            for id in table_ids
+            if id in all_data_dict.keys()  # BUG: should I warn if the id isn't found?
+        ]
+
+        return all_data_ordered
+
+    def get_table(table_name: str):
+
+        # This method can be return ANY table, so we need to import all of them
+        # here. This is a local import to prevent circular import issues.
+        from simmate.website.third_parties import models as third_party_datatables
+        from simmate.website.workflows import models as all_datatables
+
+        # Import the datatable class -- how this is done depends on if it
+        # is from a simmate supplied class or if the user supplied a full
+        # path to the class
+        # OPTIMIZE: is there a better way to do this?
+        if hasattr(all_datatables, table_name):
+            datatable = getattr(all_datatables, table_name)
+        elif hasattr(third_party_datatables, table_name):
+            datatable = getattr(third_party_datatables, table_name)
+        else:
+            datatable = import_string(table_name)
+
+        return datatable
+
     # -------------------------------------------------------------------------
     # Methods for loading results from files
     # -------------------------------------------------------------------------
