@@ -10,11 +10,23 @@ class Forces(DatabaseTable):
     class Meta:
         abstract = True
 
-    base_info = [
+    exclude_from_summary = [
         "site_forces",
         "lattice_stress",
     ]
 
+    archive_fields = [
+        "site_forces",
+        "lattice_stress",
+    ]
+
+    api_filters = dict(
+        site_force_norm_max=["range"],
+        site_forces_norm_per_atom=["range"],
+        lattice_stress_norm=["range"],
+        lattice_stress_norm_per_atom=["range"],
+    )
+
     site_forces = table_column.JSONField(blank=True, null=True)
     """
     A list of forces for each atomic site (eV/AA). So this is a list like...
@@ -64,7 +76,7 @@ class Forces(DatabaseTable):
     @classmethod
     def _from_toolkit(
         cls,
-        structure: ToolkitStructure,
+        structure: ToolkitStructure | str = None,
         site_forces=None,
         lattice_stress=None,
         as_dict=False,
@@ -73,15 +85,24 @@ class Forces(DatabaseTable):
         Given site forces and lattice stress, this function builds the rest of
         the required fields for this class as a dictionary.
         """
+
+        if isinstance(structure, str):
+            structure = ToolkitStructure.from_database_string(structure)
+
         # TODO: in the future, this should accept an IonicStep toolkit object
         # or maybe Structure + Forces toolkit objects.
         site_data = (
             dict(
                 site_forces=site_forces,
-                site_force_norm_max=max([numpy.linalg.norm(f) for f in site_forces]),
-                site_forces_norm=numpy.linalg.norm(site_forces),
-                site_forces_norm_per_atom=numpy.linalg.norm(site_forces)
-                / structure.num_sites,
+                site_force_norm_max=float(
+                    max([numpy.linalg.norm(f) for f in site_forces])
+                ),
+                site_forces_norm=float(numpy.linalg.norm(site_forces)),
+                site_forces_norm_per_atom=float(
+                    numpy.linalg.norm(site_forces) / structure.num_sites
+                )
+                if structure
+                else None,
             )
             if site_forces
             else {}
@@ -90,9 +111,12 @@ class Forces(DatabaseTable):
         lattice_data = (
             dict(
                 lattice_stress=lattice_stress,
-                lattice_stress_norm=numpy.linalg.norm(lattice_stress),
-                lattice_stress_norm_per_atom=numpy.linalg.norm(lattice_stress)
-                / structure.num_sites,
+                lattice_stress_norm=float(numpy.linalg.norm(lattice_stress)),
+                lattice_stress_norm_per_atom=float(
+                    numpy.linalg.norm(lattice_stress) / structure.num_sites
+                )
+                if structure
+                else None,
             )
             if lattice_stress
             else {}
