@@ -1,18 +1,53 @@
 # -*- coding: utf-8 -*-
 
+from django_filters import rest_framework as django_api_filters
 from scipy.constants import Avogadro
 
 from simmate.database.base_data_types import DatabaseTable, Spacegroup, table_column
 from simmate.toolkit import Structure as ToolkitStructure
+from simmate.utilities import get_chemical_subsystems
 
 
 class Structure(DatabaseTable):
     class Meta:
         abstract = True
 
-    base_info = ["structure_string"]
+    exclude_from_summary = ["structure", "elements"]
 
-    structure_string = table_column.TextField(blank=True, null=True)
+    archive_fields = ["structure"]
+
+    api_filters = dict(
+        nsites=["range"],
+        nelements=["range"],
+        # elements=["contains"],
+        density=["range"],
+        density_atomic=["range"],
+        volume=["range"],
+        volume_molar=["range"],
+        formula_full=["exact"],
+        formula_reduced=["exact"],
+        formula_anonymous=["exact"],
+        spacegroup__number=["exact"],
+        spacegroup__symbol=["exact"],
+        spacegroup__crystal_system=["exact"],
+        spacegroup__point_group=["exact"],
+        # Whether to include subsystems of the given `chemical_system`. For
+        # example, the subsystems of Y-C-F would be Y, C, F, Y-C, Y-F, etc..
+        include_subsystems=django_api_filters.BooleanFilter(
+            field_name="include_subsystems",
+            label="Include chemical subsystems in results?",
+            method="skip_filter",
+        ),
+        # TODO: Supra-systems would include all the elements listed AND more. For example,
+        # searching Y-C-F would also return Y-C-F-Br, Y-Sc-C-F, etc.
+        # include_suprasystems = forms.BooleanField(label="Include Subsytems", required=False)
+        # The chemical system of the structure (e.g. "Y-C-F" or "Na-Cl")
+        chemical_system=django_api_filters.CharFilter(
+            method="filter_chemical_system",
+        ),
+    )
+
+    structure = table_column.TextField(blank=True, null=True)
     """
     The core structure information, which is written to a string and in a 
     compressed format using the `from_toolkit` method. To get back to our toolkit
@@ -127,14 +162,53 @@ class Structure(DatabaseTable):
     # criteria, you can still do this in python and pandas! Just not at the
     # SQL level
 
+    def filter_chemical_system(self, queryset, name, value):
+        # name/value here are the key/value pair for chemical system
+
+        # Grab the "include_subsystems" field from the filter form. Note, this
+        # value will be given as a string which we convert to a python boolean
+        include_subsystems = self.data.dict().get("include_subsystems", "false")
+        include_subsystems = True if include_subsystems == "true" else False
+
+        # TODO:
+        # Make sure that the chemical system is made of valid elements and
+        # separated by hyphens
+
+        # check if the user wants subsystems included (This will be True or False)
+        if include_subsystems:
+            systems_cleaned = get_chemical_subsystems(value)
+
+        # otherwise just clean the single system
+        else:
+            # Convert the system to a list of elements
+            systems_cleaned = value.split("-")
+            # now recombine the list back into alphabetical order
+            systems_cleaned = ["-".join(sorted(systems_cleaned))]
+            # NOTE: we call this "systems_cleaned" and put it in a list so
+            # that our other methods don't have to deal with multiple cases
+            # when running a django filter.
+
+        filtered_queryset = queryset.filter(chemical_system__in=systems_cleaned)
+
+        # now return the cleaned value. Note that this is now a list of
+        # chemical systems, where all elements are in alphabetical order.
+        return filtered_queryset
+
     @classmethod
     def _from_toolkit(
         cls,
-        structure: ToolkitStructure,
+        structure: ToolkitStructure | str = None,
         as_dict: bool = False,
         **kwargs,
     ):
 
+        if isinstance(structure, str):
+            structure = ToolkitStructure.from_database_string(structure)
+
+        # if there isn't a structure, nothing is to be done.
+        if not structure:
+            return kwargs if as_dict else cls(**kwargs)
+
         # BUG: This is an old line and I can't remember why I have it. Once I
         # have implemented more unittests, consider deleting. This method is
         # ment to convert to a ToolkitStructure, but the structure should
@@ -156,17 +230,19 @@ class Structure(DatabaseTable):
         # from pymatgen.analysis.prototypes import AflowPrototypeMatcher
         # prototype = AflowPrototypeMatcher().get_prototypes(structure)
         # prototype_name = prototype[0]["tags"]["mineral"] if prototype else None
+        # Alternatively, add as a method to the table, similar to
+        # the "update_all_stabilities" for thermodynamics
 
         # Given a pymatgen structure object, this will return a database structure
         # object, but will NOT save it to the database yet. The kwargs input
         # is only if you inherit from this class and add extra fields.
         structure_dict = dict(
-            structure_string=structure.to(fmt=storage_format),
+            structure=structure.to(fmt=storage_format),
             nsites=structure.num_sites,
             nelements=len(structure.composition),
             elements=[str(e) for e in structure.composition.elements],
             chemical_system=structure.composition.chemical_system,
-            density=structure.density,
+            density=float(structure.density),
             density_atomic=structure.num_sites / structure.volume,
             volume=structure.volume,
             # 1e-27 is to convert from cubic angstroms to Liter and then 1e3 to
