@@ -16,8 +16,9 @@ class AflowStructure(Structure, Thermodynamics):
     class Meta:
         app_label = "third_parties"
 
-    base_info = ["id", "structure_string", "energy"]
     source = "AFLOW"
+    source_long = "The Automatic-FLOW for Materials Discovery"
+    homepage = "http://www.aflowlib.org/"
     source_doi = "https://doi.org/10.1016/j.commatsci.2012.02.005"
 
     id = table_column.CharField(max_length=25, primary_key=True)
