@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
 
+import itertools
+
 import pandas as pd
 from numpy.random import choice, shuffle
-import itertools
 
-from simmate.toolkit import Structure, Composition
-
-from simmate.toolkit.symmetry.wyckoff import loadWyckoffData
-from simmate.toolkit.creators.vector import UniformlyDistributedVectors
-from simmate.toolkit.creators.sites.random_wyckoff import (
-    asymmetric_unit_boundries,
-)  #!!! this will move to simmate.toolkit.symmetry in the future
+from simmate.toolkit import Composition, Structure
 from simmate.toolkit.creators.lattice import RSLSmartVolume
+from simmate.toolkit.creators.sites.random_wyckoff import (  # !!! this will move to simmate.toolkit.symmetry in the future
+    asymmetric_unit_boundries,
+)
+from simmate.toolkit.creators.vector import UniformlyDistributedVectors
+from simmate.toolkit.symmetry.wyckoff import loadWyckoffData
 
 # from pymatdisc.core.estimate import distance_matrix # !!! TODO
 
