@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 
-from pymatgen.analysis.diffusion.neb.pathfinder import DistinctPathFinder
-
+from .distinct_path_finder import DistinctPathFinder
 from .migration_hop import MigrationHop
 from .migration_images import MigrationImages
