@@ -51,8 +51,8 @@ class MigrationHop(PymatgenMigrationHop):
 
         ex:
             {
-                "migration_hop_table": "MITMigrationHop",
-                "migration_hop_id": 1,
+                "database_table": "MITMigrationHop",
+                "database_id": 1,
             }
         """
 
@@ -62,16 +62,16 @@ class MigrationHop(PymatgenMigrationHop):
         from simmate.database import connect
         from simmate.website.workflows import models as all_datatables
 
-        datatable_str = migration_hop["migration_hop_table"]
+        datatable_str = migration_hop["database_table"]
 
         if hasattr(all_datatables, datatable_str):
             datatable = getattr(all_datatables, datatable_str)
         else:
             datatable = import_string(datatable_str)
         # for now I only support migration_hop_id
-        migration_hop_db = datatable.objects.get(id=migration_hop["migration_hop_id"])
-        migration_hop_cleaned = migration_hop_db.to_toolkit()
-        migration_hop_cleaned.database_entry = migration_hop_db
+        migration_hop_db = datatable.objects.get(id=migration_hop["database_id"])
+        migration_hop_cleaned = migration_hop_db.to_migration_hop_toolkit()
+        migration_hop_cleaned.database_object = migration_hop_db
 
         return migration_hop_cleaned
 
