@@ -5,12 +5,16 @@ import math
 import traceback
 from pathlib import Path
 
+import numpy
 import pandas
+import plotly.express as plotly_express
 import plotly.graph_objects as plotly_go
 from rich.progress import track
 
+from simmate.configuration.dask import get_dask_client
 from simmate.database.base_data_types import Calculation, table_column
 from simmate.toolkit import Composition, Structure
+from simmate.toolkit.structure_prediction.evolution import selectors as selector_module
 from simmate.toolkit.structure_prediction.evolution.database import SteadystateSource
 from simmate.toolkit.validators import fingerprint as validator_module
 from simmate.utilities import get_directory
@@ -56,6 +60,14 @@ class FixedCompositionSearch(Calculation):
     # the time to sleep between file writing and steady-state checks.
     sleep_step = table_column.FloatField(null=True, blank=True)
 
+    # Loading all unique structures is an expensive operation so its only
+    # done on a cycle. This stores the list of unique ids.
+    unique_individuals_ids = table_column.JSONField(
+        default=list,
+        null=True,
+        blank=True,
+    )
+
     # This is an optional an input for an expected structure in order to allow
     # creation of plots that show convergence vs. the expected. This is very
     # useful benchmarking and when the user has a target structure that they
@@ -152,7 +164,7 @@ class FixedCompositionSearch(Calculation):
         # assume we are using energy_per_atom
         num_new_structures_since_best = self.individuals.filter(
             energy_per_atom__gt=best.energy_per_atom + self.convergence_cutoff,
-            created_at__gte=best.created_at,
+            finished_at__gte=best.finished_at,
         ).count()
         if num_new_structures_since_best > self.best_survival_cutoff:
             logging.info(
@@ -456,6 +468,34 @@ class FixedCompositionSearch(Calculation):
     def get_nbest_indiviudals(self, nbest: int):
         return self.individuals_completed.order_by(self.fitness_field)[:nbest]
 
+    def get_unique_individuals(
+        self,
+        use_cache: bool = False,
+        as_queryset: bool = False,
+    ):
+
+        # get the most-up-to-date results but is slow
+        if not use_cache:
+            unique = self.validator.get_unique_from_pool()
+            # This is an expensive method as the calculation scales, so make sure
+            # we cache thes values
+            self.unique_individuals_ids = [s.database_object.id for s in unique]
+            self.save()
+
+        # OPTIMIZE: this converts back to a queryset object but involves
+        # another database query.
+        if as_queryset or use_cache:
+            unique = (
+                self.individuals_completed.filter(id__in=self.unique_individuals_ids)
+                .order_by(self.fitness_field)
+                .all()
+            )
+
+        if not as_queryset and not isinstance(unique, list):
+            unique = unique.to_toolkit()
+
+        return unique
+
     def get_best_individual_history(self):
         """
         Goes through all structures in order that they were created and creates
@@ -463,7 +503,7 @@ class FixedCompositionSearch(Calculation):
         """
 
         individuals = (
-            self.individuals_completed.order_by("updated_at")
+            self.individuals_completed.order_by("finished_at")
             .only("id", self.fitness_field)
             .all()
         )
@@ -482,7 +522,7 @@ class FixedCompositionSearch(Calculation):
 
         return best_history
 
-    def write_summary(self, directory: Path):
+    def write_output_summary(self, directory: Path):
         logging.info(f"Writing search summary to {directory}")
 
         super().write_output_summary(directory=directory)
@@ -492,8 +532,11 @@ class FixedCompositionSearch(Calculation):
         # error here -- but instead warn the user and then continue the search
         try:
             # calls all the key methods defined below
-            best_cifs_directory = get_directory(directory / "best_structures_cifs")
+            best_cifs_directory = get_directory(directory / "best_structures")
             self.write_best_structures(100, best_cifs_directory)
+            unique_cifs_directory = get_directory(directory / "best_structures_unique")
+            self.write_unique_structures(unique_cifs_directory)
+
             self.write_individuals_completed(directory=directory)
             self.write_individuals_completed_full(directory=directory)
             self.write_best_individuals_history(directory=directory)
@@ -504,21 +547,9 @@ class FixedCompositionSearch(Calculation):
 
             # BUG: This is only for "relaxation.vasp.staged", which the assumed
             # workflow for now.
-            composition = Composition(self.composition)
-            self.subworkflow.write_staged_series_convergence_plot(
-                directory=directory,
-                # See `individuals` method for why we use these filters
-                formula_reduced=composition.reduced_formula,
-                nsites__lte=composition.num_atoms,
-                energy_per_atom__isnull=False,
-            )
-            self.subworkflow.write_staged_series_histogram_plot(
-                directory=directory,
-                # See `individuals` method for why we use these filters
-                formula_reduced=composition.reduced_formula,
-                nsites__lte=composition.num_atoms,
-                energy_per_atom__isnull=False,
-            )
+            self.write_staged_series_convergence_plot(directory=directory)
+            self.write_staged_series_histogram_plot(directory=directory)
+            self.write_staged_series_times_plot(directory=directory)
 
             logging.info("Done writing summary.")
 
@@ -539,15 +570,14 @@ class FixedCompositionSearch(Calculation):
     @property
     def selector(self):
 
-        # Initialize the selector
-        if self.selector_name == "TruncatedSelection":
-            from simmate.toolkit.structure_prediction.evolution.selectors import (
-                TruncatedSelection,
-            )
+        # !!! This is largley a copy/paste of the validator method. Consider
+        # making a utility to load classes by name
 
-            selector = TruncatedSelection()
-        else:  # BUG
-            raise Exception("We only support TruncatedSelection right now")
+        if hasattr(selector_module, self.selector_name):
+            selector_class = getattr(selector_module, self.selector_name)
+            selector = selector_class(**self.selector_kwargs)
+        else:
+            raise Exception("Unknown selector name provided")
 
         return selector
 
@@ -562,16 +592,18 @@ class FixedCompositionSearch(Calculation):
         logging.info("Generating fingerprints for past structures...")
         fingerprint_validator = validator_class(
             composition=Composition(self.composition),
-            structure_pool=self.individuals,
+            structure_pool=self.individuals_completed.order_by(self.fitness_field),
             **self.validator_kwargs,
         )
         logging.info("Done generating fingerprints.")
 
-        # BUG: should we only do structures that were successfully calculated?
+        # for now I only include final structures that have been calculated
+        # OPTIMIZE: should we only do structures that were successfully calculated?
         # If not, there's a chance a structure fails because of something like a
         # crashed slurm job, but it's never submitted again...
-        # OPTIMIZE: should we only do final structures? Or should I include input
+        # Or should we only do final structures? Or should I include input
         # structures and even all ionic steps as well...?
+
         return fingerprint_validator
 
     # -------------------------------------------------------------------------
@@ -579,6 +611,16 @@ class FixedCompositionSearch(Calculation):
     # -------------------------------------------------------------------------
 
     def write_best_structures(self, nbest: int, directory: Path):
+        best = self.get_nbest_indiviudals(nbest)
+        structures = best.only("structure", "id").to_toolkit()
+        self._write_structures(structures, directory)
+
+    def write_unique_structures(self, directory: Path):
+        structures = self.get_unique_individuals(use_cache=False)
+        self._write_structures(structures, directory)
+
+    # TODO: consider making a utility elsewhere
+    def _write_structures(self, structures: list[Structure], directory: Path):
         # if the directory is filled, we need to delete all the files
         # before writing the new ones.
         for file in directory.iterdir():
@@ -587,14 +629,12 @@ class FixedCompositionSearch(Calculation):
             except OSError:
                 logging.warning("Unable to delete a CIF file: {file}")
                 logging.warning(
-                    "Updating the 'best structures' directory involves deleting "
+                    "Updating this directory involves deleting "
                     "and re-writing all CIF files each cycle. If you have a file "
                     "open while this step occurs, then you'll see this warning."
                     "Close your file for this to go away."
                 )
 
-        best = self.get_nbest_indiviudals(nbest)
-        structures = best.only("structure", "id").to_toolkit()
         for rank, structure in enumerate(structures):
             rank_cleaned = str(rank).zfill(2)  # converts 1 to 01
             structure_filename = (
@@ -614,7 +654,7 @@ class FixedCompositionSearch(Calculation):
         columns = [
             "id",
             "energy_per_atom",
-            "updated_at",
+            "finished_at",
             "source",
             "spacegroup__number",
         ]
@@ -630,7 +670,7 @@ class FixedCompositionSearch(Calculation):
         def format_date(date):
             return date.strftime("%Y-%m-%d %H:%M:%S")
 
-        df["updated_at"] = df.updated_at.apply(format_date)
+        df["finished_at"] = df.finished_at.apply(format_date)
 
         def format_parents(source):
             return source.get("parent_ids", None) if source else None
@@ -653,11 +693,11 @@ class FixedCompositionSearch(Calculation):
         df.to_markdown(md_filename)
 
     def write_best_individuals_history(self, directory: Path):
-        columns = ["id", "energy_per_atom", "updated_at"]
+        columns = ["id", "energy_per_atom", "finished_at"]
         best_history = self.get_best_individual_history()
         df = (
             self.individuals.filter(id__in=best_history)
-            .order_by("-updated_at")
+            .order_by("-finished_at")
             .only(*columns)
             .to_dataframe(columns)
         )
@@ -666,7 +706,7 @@ class FixedCompositionSearch(Calculation):
         def format_date(date):
             return date.strftime("%Y-%m-%d %H:%M:%S")
 
-        df["updated_at"] = df.updated_at.apply(format_date)
+        df["finished_at"] = df.finished_at.apply(format_date)
         md_filename = directory / "history_of_the_best_individuals.md"
         df.to_markdown(md_filename)
 
@@ -703,19 +743,19 @@ class FitnessConvergence(PlotlyFigure):
     def get_plot(search: FixedCompositionSearch):
 
         # Grab the calculation's structure and convert it to a dataframe
-        columns = ["updated_at", search.fitness_field]
+        columns = ["finished_at", search.fitness_field]
         structures_dataframe = search.individuals_completed.only(*columns).to_dataframe(
             columns
         )
 
         # There's only one plot here, no subplot. So we make the scatter
         # object and just pass it directly to a Figure object
-        scatter = plotly_go.Scatter(
-            x=structures_dataframe["updated_at"],
+        figure = plotly_express.scatter(
+            x=structures_dataframe["finished_at"],
             y=structures_dataframe[search.fitness_field],
-            mode="markers",
+            marginal_x="histogram",
+            marginal_y="histogram",
         )
-        figure = plotly_go.Figure(data=scatter)
 
         figure.update_layout(
             xaxis_title="Date Completed",
@@ -733,29 +773,8 @@ class Correctness(PlotlyFigure):
         structure_known: Structure,
     ):
 
-        # --------------------------------------------------------
-        # This code is from simmate.toolkit.validators.fingerprint.pcrystalnn
-        # OPTIMIZE: There should be a convience method to make this featurizer
-        # since I use it so much
-        import numpy
-        from matminer.featurizers.site import CrystalNNFingerprint
-        from matminer.featurizers.structure.sites import (  # PartialsSiteStatsFingerprint,
-            SiteStatsFingerprint,
-        )
-
-        from simmate.toolkit import Composition
-
-        sitefingerprint_method = CrystalNNFingerprint.from_preset(
-            "ops", distance_cutoffs=None, x_diff_weight=3
-        )
-        featurizer = SiteStatsFingerprint(
-            sitefingerprint_method,
-            stats=["mean", "std_dev", "minimum", "maximum"],
-        )
-        featurizer.elements_ = numpy.array(
-            [element.symbol for element in Composition(search.composition).elements]
-        )
-        # --------------------------------------------------------
+        # load the featurizer from the search object
+        featurizer = search.validator.featurizer
 
         # Grab the calculation's structure and convert it to a dataframe
         structures_dataframe = search.individuals_completed.to_dataframe()
@@ -772,29 +791,51 @@ class Correctness(PlotlyFigure):
         # through the queryset like in the commented out code above. Instead,
         # we need to iterate through the dataframe rows.
         # See https://github.com/chrisdev/django-pandas/issues/138 for issue
-        from simmate.toolkit import Structure
 
         structures_dataframe["structure"] = [
             Structure.from_str(s.structure, fmt="POSCAR")
             for _, s in structures_dataframe.iterrows()
         ]
 
-        structures_dataframe["fingerprint"] = [
-            numpy.array(featurizer.featurize(s.structure))
-            for _, s in track(structures_dataframe.iterrows())
+        # generating fingerprints is slow so we use dask to parallelize
+        client = get_dask_client()
+        logging.info("Submitting to jobs dask...")
+        futures = [
+            client.submit(featurizer.featurize, s.structure, pure=False)
+            for _, s in track(list(structures_dataframe.iterrows()))
         ]
+        logging.info("Waiting for dask jobs to finish...")
+        structures_dataframe["fingerprint"] = [numpy.array(f.result()) for f in futures]
+        logging.info("Done.")
 
         fingerprint_known = numpy.array(featurizer.featurize(structure_known))
 
-        structures_dataframe["fingerprint_distance"] = [
-            numpy.linalg.norm(fingerprint_known - s.fingerprint)
-            for _, s in track(structures_dataframe.iterrows())
-        ]
+        distances = []
+        smallest_distance_id = None
+        smallest_distance = 999
+        for _, s in structures_dataframe.iterrows():
+
+            distance = numpy.linalg.norm(fingerprint_known - s.fingerprint)
+            distances.append(distance)
+
+            # import scipy
+            # scipy.spatial.distance.cosine(fingerprint_known, s.fingerprint)
+            # BUG: I assume distance method. I need to check the validator in
+            # case the method prefers something like cosine distance.
+
+            if distance < smallest_distance:
+                smallest_distance = distance
+                smallest_distance_id = s.id
+        logging.info(
+            f"The most similar structure is id={smallest_distance_id} "
+            f"with distance {smallest_distance:.3f}"
+        )
+        structures_dataframe["fingerprint_distance"] = distances
 
         # There's only one plot here, no subplot. So we make the scatter
         # object and just pass it directly to a Figure object
         scatter = plotly_go.Scatter(
-            x=structures_dataframe["updated_at"],
+            x=structures_dataframe["finished_at"],
             y=structures_dataframe["fingerprint_distance"],
             mode="markers",
             marker_color=structures_dataframe["energy_per_atom"],
@@ -813,21 +854,25 @@ class SubworkflowTimes(PlotlyFigure):
     def get_plot(search: FixedCompositionSearch):
 
         # Grab the calculation's structure and convert it to a dataframe
-        columns = ["created_at", "updated_at"]
-        df = search.individuals_completed.only(*columns).to_dataframe(columns)
-        df["total_time"] = df.updated_at - df.created_at
+        data = search.individuals_completed.values_list(
+            "total_time", "queue_time"
+        ).all()
 
-        def convert_to_min(time):
-            # time is stored in nanoseconds and we convert to minutes
-            return time.value * (10**-9) / 60
+        # time is stored in seconds and we convert to minutes
+        total_times = [e[0] / 60 for e in data]
+        # queue_times = [e[1] / 60 for e in data]
 
-        df["total_time_min"] = df.total_time.apply(convert_to_min)
-        histogram = plotly_go.Histogram(x=df.total_time_min)
-        figure = plotly_go.Figure(data=histogram)
+        figure = plotly_go.Figure()
+        hist_1 = plotly_go.Histogram(x=total_times)  # , name="Total run time (min)"
+        # hist_2 = plotly_go.Histogram(x=queue_times, name="Total queue time (min)")
+        figure.add_trace(hist_1)
+        # figure.add_trace(hist_2)
         figure.update_layout(
-            xaxis_title="Total time (min)",
+            xaxis_title="Total calculation time (min)",
             yaxis_title="Individuals (#)",
+            barmode="overlay",
         )
+        # figure.update_traces(opacity=0.75)
         return figure
 
 
@@ -856,11 +901,50 @@ class FitnessDistribution(PlotlyFigure):
         return figure
 
 
+class StagedSeriesConvergence(PlotlyFigure):
+    def get_plot(search: FixedCompositionSearch):
+        composition = Composition(search.composition)
+        plot = search.subworkflow.get_staged_series_convergence_plot(
+            # See `individuals` method for why we use these filters
+            formula_reduced=composition.reduced_formula,
+            nsites__lte=composition.num_atoms,
+            energy_per_atom__isnull=False,
+        )
+        return plot
+
+
+class StagedSeriesHistogram(PlotlyFigure):
+    def get_plot(search: FixedCompositionSearch):
+        composition = Composition(search.composition)
+        plot = search.subworkflow.get_staged_series_histogram_plot(
+            # See `individuals` method for why we use these filters
+            formula_reduced=composition.reduced_formula,
+            nsites__lte=composition.num_atoms,
+            energy_per_atom__isnull=False,
+        )
+        return plot
+
+
+class StagedSeriesTimes(PlotlyFigure):
+    def get_plot(search: FixedCompositionSearch):
+        composition = Composition(search.composition)
+        plot = search.subworkflow.get_staged_series_times_plot(
+            # See `individuals` method for why we use these filters
+            formula_reduced=composition.reduced_formula,
+            nsites__lte=composition.num_atoms,
+            energy_per_atom__isnull=False,
+        )
+        return plot
+
+
 # register all plotting methods to the database table
 for _plot in [
     FitnessConvergence,
     Correctness,
     FitnessDistribution,
     SubworkflowTimes,
+    StagedSeriesConvergence,
+    StagedSeriesHistogram,
+    StagedSeriesTimes,
 ]:
     _plot.register_to_class(FixedCompositionSearch)
