@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
 from .atomic_permutation import AtomicPermutation
-from .coordinate_perturation_ordered import CoordinateOrderedPerturbation
 from .coordinate_perturbation import CoordinatePerturbation
+from .extreme_symmetry import ExtremeSymmetry
 from .lattice_strain import LatticeStrain
