@@ -4,4 +4,4 @@
 # they are located at. I do this based on the directions given by:
 # https://docs.djangoproject.com/en/3.1/topics/db/models/#organizing-models-in-a-package
 
-from simmate.database.base_data_types import Spacegroup
+from simmate.database.base_data_types import Fingerprint, FingerprintPool, Spacegroup
