@@ -37,13 +37,11 @@ def get_all_workflows(
         try:
             app_module = importlib.import_module(f"{app_modulename}.workflows")
         except Exception as error:
-            logging.warning(
+            logging.critical(
                 f"Failed to load workflows from {app_name}. Did you make sure "
                 "there is a workflows.py file or module present?"
-                f"The error given was...\n {error}"
             )
-            # raise error  --- should I just raise the error?
-            continue
+            raise error
 
         # iterate through each available object in the workflows file and find
         # which ones are workflow objects.
@@ -129,6 +127,7 @@ def get_workflow_names_by_type(
     calculator_name: str = None,
     full_name: bool = True,
     precheck_type_exists: bool = True,
+    remove_no_database_flows: bool = False,
 ) -> list[str]:
     """
     Returns a list of all the workflows of a given type. Optionally, the
@@ -153,6 +152,9 @@ def get_workflow_names_by_type(
         if calculator_name and flow.name_calculator != calculator_name:
             continue  # Skip those that don't match
 
+        if remove_no_database_flows and not flow.use_database:
+            continue
+
         if full_name:
             workflow_name = flow.name_full
         else:
